import pytest

# don't use `pytest.main()` anymore! we're getting problems here due to product_test db entity.
# please use `pytest.main(['-x', 'exporter/tests/' , 'common/tests/'])` instead!
# pytest.main(['-x', 'exporter/tests/'])
pytest.main(['-x', 'tests'])



# -*- coding: utf-8 -*-
"""
Copyright 2019 eBay Inc.
 
Licensed under the Apache License, Version 2.0 (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,

WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

See the License for the specific language governing permissions and
limitations under the License.
"""
import json
import logging

import yaml

from .model.model import Environment, Credentials

user_config_ids = ["sandbox-user", "production-user"]


class Credentialutil(object):
    """
    credential_list: dictionary key=string, value=Credentials
    """
    _credential_list = {}

    @classmethod
    def load(cls, app_config_path):
        logging.info("Loading credential configuration file at: %s", app_config_path)
        with open(app_config_path, 'r') as f:
            if app_config_path.endswith('.yaml') or app_config_path.endswith('.yml'):
                content = yaml.load(f, Loader=yaml.FullLoader)
            elif app_config_path.endswith('.json'):
                content = json.loads(f.read())
            else:
                raise ValueError('Configuration file need to be in JSON or YAML')
            Credentialutil._iterate(content)

    @classmethod
    def _iterate(cls, content):
        for key in content:
            logging.debug("Environment attempted: %s", key)

            if key in [Environment.PRODUCTION.config_id, Environment.SANDBOX.config_id]:
                client_id = content[key]['appid']
                dev_id = content[key]['devid']
                client_secret = content[key]['certid']
                ru_name = content[key]['redirecturi']

                app_info = Credentials(client_id, client_secret, dev_id, ru_name)
                cls._credential_list.update({key: app_info})

    @classmethod
    def get_credentials(cls, env_type):
        """
        env_config_id: Environment.PRODUCTION.config_id or Environment.SANDBOX.config_id
        """
        if len(cls._credential_list) == 0:
            msg = "No Environment loaded from configuration file"
            logging.error(msg)
            raise CredentialNotLoadedError(msg)
        return cls._credential_list[env_type.config_id]


class CredentialNotLoadedError(Exception):
    pass



# -*- coding: utf-8 -*-
"""
Copyright 2019 eBay Inc.
 
Licensed under the Apache License, Version 2.0 (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,

WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

See the License for the specific language governing permissions and
limitations under the License.

"""
import base64


def generate_request_headers(credential):
    cred = credential.client_id + ':' + credential.client_secret

    b64_encoded_credential = base64.b64encode(cred.encode()).decode()
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Basic ' + b64_encoded_credential
    }

    return headers


def generate_application_request_body(credential, scopes):
    body = {
        'grant_type': 'client_credentials',
        'redirect_uri': credential.ru_name,
        'scope': scopes
    }

    return body


def generate_refresh_request_body(scopes, refresh_token):
    if refresh_token is None:
        raise Exception("credential object does not contain refresh_token and/or scopes")

    body = {
        'grant_type': 'refresh_token',
        'refresh_token': refresh_token,
        'scope': scopes
    }

    return body


def generate_oauth_request_body(credential, code):
    body = {
        'grant_type': 'authorization_code',
        'redirect_uri': credential.ru_name,
        'code': code
    }

    return body



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

import logging
import sys

LOGGING_FORMAT = '%(asctime)s - [%(levelname)s] - %(message)s'


def setup_logging():
    stdout_handler = logging.StreamHandler(sys.stdout)
    handlers = [stdout_handler]
    logging.basicConfig(handlers=handlers, level=logging.DEBUG, format=LOGGING_FORMAT)



# **************************************************************************
# Copyright 2018-2019 eBay Inc.
# Author/Developers: --

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#   https://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# **************************************************************************/

from aenum import Enum, unique


@unique
class FileEncoding(Enum):
    UTF8 = 'UTF-8'

    def __str__(self):
        return str(self.value)


@unique
class FileFormat(Enum):
    GZIP = 'gzip'

    def __str__(self):
        return str(self.value)



# -*- coding: utf-8 -*-
"""
Copyright 2019 eBay Inc.
 
Licensed under the Apache License, Version 2.0 (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,

WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import logging
import urllib.error
import urllib.parse
import urllib.request
from datetime import datetime, timedelta

import requests

from ebay_feedsdk.oauthclient.credentialutil import Credentialutil
from ebay_feedsdk.oauthclient.model import util
from ebay_feedsdk.oauthclient.model.model import OauthToken

LOGFILE = 'eBay_Oauth_log.txt'
logging.basicConfig(level=logging.DEBUG, filename=LOGFILE,
                    format="%(asctime)s: %(levelname)s - %(funcName)s: %(message)s", filemode='w')


class Oauth2api(object):

    def generate_user_authorization_url(self, env_type, scopes, state=None):
        '''
            EnvType = Environment.SANDBOX or Environment.PRODUCTION
            scopes = list of strings
        '''

        credential = Credentialutil.get_credentials(env_type)

        scopes = ' '.join(scopes)
        param = {
            'client_id': credential.client_id,
            'redirect_uri': credential.ru_name,
            'response_type': 'code',
            'prompt': 'login',
            'scope': scopes
        }

        if state != None:
            param.update({'state': state})

        query = urllib.parse.urlencode(param)
        return env_type.web_endpoint + '?' + query

    def get_application_token(self, env_type, scopes):
        """
            makes call for application token and stores result in credential object
            returns credential object
        """

        logging.info("Trying to get a new application access token ... ")
        credential = Credentialutil.get_credentials(env_type)
        headers = util.generate_request_headers(credential)
        body = util.generate_application_request_body(credential, ' '.join(scopes))

        resp = requests.post(env_type.api_endpoint, data=body, headers=headers)
        content = json.loads(resp.content)
        token = OauthToken()

        if resp.status_code == requests.codes.ok:
            token.access_token = content['access_token']
            # set token expiration time 5 minutes before actual expire time
            token.token_expiry = datetime.utcnow() + timedelta(seconds=int(content['expires_in'])) - timedelta(
                minutes=5)

        else:
            token.error = str(resp.status_code) + ': ' + content['error_description']
            logging.error("Unable to retrieve token.  Status code: %s - %s", resp.status_code,
                          requests.status_codes._codes[resp.status_code])
            logging.error("Error: %s - %s", content['error'], content['error_description'])

        return token

    def exchange_code_for_access_token(self, env_type, code):
        logging.info("Trying to get a new user access token ... ")
        credential = Credentialutil.get_credentials(env_type)

        headers = util.generate_request_headers(credential)
        body = util.generate_oauth_request_body(credential, code)
        resp = requests.post(env_type.api_endpoint, data=body, headers=headers)

        content = json.loads(resp.content)
        token = OauthToken()

        if resp.status_code == requests.codes.ok:
            token.access_token = content['access_token']
            token.token_expiry = datetime.utcnow() + timedelta(seconds=int(content['expires_in'])) - timedelta(
                minutes=5)
            token.refresh_token = content['refresh_token']
            token.refresh_token_expiry = datetime.utcnow() + timedelta(
                seconds=int(content['refresh_token_expires_in'])) - timedelta(minutes=5)
        else:
            token.error = str(resp.status_code) + ': ' + content['error_description']
            logging.error("Unable to retrieve token.  Status code: %s - %s", resp.status_code,
                          requests.status_codes._codes[resp.status_code])
            logging.error("Error: %s - %s", content['error'], content['error_description'])
        return token

    def get_access_token(self, env_type, refresh_token, scopes):
        """
        refresh token call
        """

        logging.info("Trying to get a new user access token ... ")

        credential = Credentialutil.get_credentials(env_type)

        headers = util.generate_request_headers(credential)
        body = util.generate_refresh_request_body(' '.join(scopes), refresh_token)
        resp = requests.post(env_type.api_endpoint, data=body, headers=headers)
        content = json.loads(resp.content)
        token = OauthToken()
        token.token_response = content

        if resp.status_code == requests.codes.ok:
            token.access_token = content['access_token']
            token.token_expiry = datetime.utcnow() + timedelta(seconds=int(content['expires_in'])) - timedelta(
                minutes=5)
        else:
            token.error = str(resp.status_code) + ': ' + content['error_description']
            logging.error("Unable to retrieve token.  Status code: %s - %s", resp.status_code,
                          requests.status_codes._codes[resp.status_code])
            logging.error("Error: %s - %s", content['error'], content['error_description'])
        return token



# -*- coding: utf-8 -*-
"""
Copyright 2019 eBay Inc.
 
Licensed under the Apache License, Version 2.0 (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,

WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

See the License for the specific language governing permissions and
limitations under the License.

"""

import os, sys
sys.path.insert(0, os.path.join(os.path.split(__file__)[0], '..'))
from oauthclient.oauth2api import Oauth2api
import test_util
from oauthclient.credentialutil import Credentialutil
from oauthclient.model.model import Environment
import unittest

app_scopes = ["https://api.ebay.com/oauth/api_scope", "https://api.ebay.com/oauth/api_scope/sell.inventory", "https://api.ebay.com/oauth/api_scope/sell.marketing", "https://api.ebay.com/oauth/api_scope/sell.account", "https://api.ebay.com/oauth/api_scope/sell.fulfillment"]

class TestGetApplicationCredential(unittest.TestCase):
    def test_generate_authorization_url(self):
        app_config_path = os.path.join(os.path.split(__file__)[0], 'config', 'ebay-config-sample-user.yaml')
        Credentialutil.load(app_config_path)
        oauth2api_inst = Oauth2api()
        signin_url = oauth2api_inst.generate_user_authorization_url(Environment.SANDBOX, app_scopes)
        self.assertIsNotNone(signin_url)
        print('\n *** test_get_signin_url ***: \n', signin_url)
    
    def test_exchange_authorization_code(self):
        app_config_path = os.path.join(os.path.split(__file__)[0], 'config', 'ebay-config-sample-user.yaml')
        Credentialutil.load(app_config_path)
        oauth2api_inst = Oauth2api()
        signin_url = oauth2api_inst.generate_user_authorization_url(Environment.SANDBOX, app_scopes)
        code = test_util.get_authorization_code(signin_url)
        user_token = oauth2api_inst.exchange_code_for_access_token(Environment.SANDBOX, code)
        self.assertIsNotNone(user_token.access_token)
        self.assertTrue(len(user_token.access_token) > 0)
        print('\n *** test_get_user_access_token ***:\n', user_token)

    def test_exchange_refresh_for_access_token(self):
        app_config_path = os.path.join(os.path.split(__file__)[0], 'config', 'ebay-config-sample-user.yaml')
        Credentialutil.load(app_config_path)
        oauth2api_inst = Oauth2api()
        signin_url = oauth2api_inst.generate_user_authorization_url(Environment.SANDBOX, app_scopes)
        code = test_util.get_authorization_code(signin_url)
        user_token = oauth2api_inst.exchange_code_for_access_token(Environment.SANDBOX, code)
        self.assertIsNotNone(user_token.refresh_token)
        self.assertTrue(len(user_token.refresh_token) > 0)
        
        user_token = oauth2api_inst.get_access_token(Environment.SANDBOX, user_token.refresh_token, app_scopes)
        self.assertIsNotNone(user_token.access_token)
        self.assertTrue(len(user_token.access_token) > 0)

        print('\n *** test_refresh_user_access_token ***:\n', user_token)

if __name__ == '__main__':
    unittest.main()


import logging

from ebay_feedsdk.errors.custom_exceptions import DownloadError
from ebay_feedsdk.feed import Feed
from ebay_feedsdk.filter.feed_filter import GetFeedResponse
from ebay_feedsdk.oauthclient.credentialutil import Credentialutil
from ebay_feedsdk.oauthclient.model.model import Environment, OauthToken, EnvType
from ebay_feedsdk.oauthclient.oauth2api import Oauth2api

class EbayDownloadExample:
    app_scopes = ["https://api.ebay.com/oauth/api_scope", "https://api.ebay.com/oauth/api_scope/buy.item.feed"]
    config_file = 'ebay-config.yaml'

    def __init__(self, market_place: str, env: EnvType, feed_scope, download_location: str):
        self.env = env
        self.feed_scope = feed_scope
        self.market_place = market_place
        self.download_location = download_location

    def download(self, category_id: str) -> GetFeedResponse:
        logging.info(
            f'Downloading category {category_id} for {self.market_place} with scope {self.feed_scope}'
            f'to {self.download_location}')

        token = self.get_token()

        feed_obj = Feed(feed_type='item', feed_scope=self.feed_scope, category_id=category_id,
                        marketplace_id=self.market_place,
                        token=token.access_token, environment=self.env.name, download_location=self.download_location)

        feed_response: GetFeedResponse = feed_obj.get()

        if feed_response.status_code != 0:
            raise DownloadError(f'Download failed see: {feed_response.message}')

        logging.info(f'File was downloaded under {feed_response.file_path}')

        return feed_response

    def get_token(self) -> OauthToken:
        Credentialutil.load(self.config_file)
        oauth2api = Oauth2api()

        token = oauth2api.get_application_token(self.env, self.app_scopes)
        if not token.access_token:
            raise DownloadError(f'Got no token, check: {token.error}')

        return token


if __name__ == "__main__":
    market_place = 'EBAY_DE'
    feed_scope = 'ALL_ACTIVE'
    download_location = '/tmp/feed'
    category_id = '2984'  # string ..
    ebay_download = EbayDownloadExample(market_place, Environment.PRODUCTION, feed_scope, download_location)
    feed_response = ebay_download.download(category_id)
    print(feed_response)




# -*- coding: utf-8 -*-
"""
Copyright 2019 eBay Inc.
 
Licensed under the Apache License, Version 2.0 (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,

WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

See the License for the specific language governing permissions and
limitations under the License.

"""
import os, logging, json, time, urllib.request, urllib.parse, urllib.error, re, yaml
from selenium import webdriver

sandbox_key = "sandbox-user"
production_key = "production-user"
_user_credential_list = {}

def read_user_info(conf = None):

    logging.info("Loading user credential configuration file at: %s", conf)
    with open(conf, 'r') as f:
        if conf.endswith('.yaml') or conf.endswith('.yml'):
            content = yaml.load(f)
        elif conf.endswith('.json'):
            content = json.loads(f.read())
        else:
            raise ValueError('Configuration file need to be in JSON or YAML')

        for key in content:
            logging.debug("Environment attempted: %s", key)
            
            if key in [sandbox_key, production_key]:       
                userid = content[key]['username']
                password = content[key]['password']        
                _user_credential_list.update({key:[userid, password]})
        
        
def get_authorization_code(signin_url):

    user_config_path = os.path.join(os.path.split(__file__)[0], 'config\\test-config-sample.yaml')
    read_user_info(user_config_path)
    
    env_key = production_key
    if "sandbox" in signin_url:
        env_key = sandbox_key
    
    userid = _user_credential_list[env_key][0]
    password = _user_credential_list[env_key][1]
    
    browser = webdriver.Chrome()
    browser.get(signin_url)
    time.sleep(5)

    form_userid = browser.find_element_by_name('userid')
    form_pw = browser.find_element_by_name('pass')  
    
    form_userid.send_keys(userid)
    form_pw.send_keys(password)    
    
    browser.find_element_by_id('sgnBt').submit()

    time.sleep(5)
    
    url = browser.current_url
    browser.quit()

    if 'code=' in url:
        code = re.findall('code=(.*?)&', url)[0]
        logging.info("Code Obtained: %s", code)
    else:
        logging.error("Unable to obtain code via sign in URL")
    
    decoded_code = urllib.parse.unquote(code).decode('utf8')
    return decoded_code
    



# -*- coding: utf-8 -*-
"""
Copyright 2019 eBay Inc.
 
Licensed under the Apache License, Version 2.0 (the "License");
You may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,

WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

See the License for the specific language governing permissions and
limitations under the License.

"""


class EnvType(object):
    def __init__(self, config_id, web_endpoint, api_endpoint, name):
        self.config_id = config_id
        self.web_endpoint = web_endpoint
        self.api_endpoint = api_endpoint
        self.name = name


class Environment(object):
    PRODUCTION = EnvType("api.ebay.com", "https://auth.ebay.com/oauth2/authorize",
                         "https://api.ebay.com/identity/v1/oauth2/token", 'PRODUCTION')
    SANDBOX = EnvType("api.sandbox.ebay.com", "https://auth.sandbox.ebay.com/oauth2/authorize",
                      "https://api.sandbox.ebay.com/identity/v1/oauth2/token", 'SANDBOX')


class Credentials(object):
    def __init__(self, client_id, client_secret, dev_id, ru_name):
        self.client_id = client_id
        self.dev_id = dev_id
        self.client_secret = client_secret
        self.ru_name = ru_name


class OauthToken(object):

    def __init__(self, error=None, access_token=None, refresh_token=None, refresh_token_expiry=None, token_expiry=None):
        """
            token_expiry: datetime in UTC
            refresh_token_expiry: datetime in UTC
        """
        self.access_token = access_token
        self.token_expiry = token_expiry
        self.refresh_token = refresh_token
        self.refresh_token_expiry = refresh_token_expiry
        self.error = error

    def __str__(self):
        token_str = '{'
        if self.error is not None:
            token_str += '"error": "' + self.error + '"'
        elif self.access_token is not None:
            token_str += '"access_token": "' + self.access_token + '", "expires_in": "' + self.token_expiry.strftime(
                '%Y-%m-%dT%H:%M:%S:%f') + '"'
            if self.refresh_token is not None:
                token_str += ', "refresh_token": "' + self.refresh_token + '", "refresh_token_expire_in": "' + \
                             self.refresh_token_expiry.strftime('%Y-%m-%dT%H:%M:%S:%f') + '"'
        token_str += '}'

        return token_str



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ebay_feedsdk',
 'ebay_feedsdk.config',
 'ebay_feedsdk.constants',
 'ebay_feedsdk.enums',
 'ebay_feedsdk.errors',
 'ebay_feedsdk.examples',
 'ebay_feedsdk.filter',
 'ebay_feedsdk.oauthclient',
 'ebay_feedsdk.oauthclient.model',
 'ebay_feedsdk.tests.oauthclient',
 'ebay_feedsdk.tests.sdk',
 'ebay_feedsdk.utils']

package_data = \
{'': ['*'],
 'ebay_feedsdk': ['sample-config/*'],
 'ebay_feedsdk.tests.oauthclient': ['config/*'],
 'ebay_feedsdk.tests.sdk': ['test-data/*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'SQLAlchemy>=1.3.8,<2.0.0',
 'aenum>=3.00,<4.0',
 'certifi>=2020.12.5,<2021.0.0',
 'pandas>=1.1.1,<2.0.0',
 'requests>=2.25.1,<3.0.0',
 'urllib3>=1.26.4,<2.0.0']

setup_kwargs = {
    'name': 'ebay-feedsdk',
    'version': '0.2.5',
    'description': 'Port of https://github.com/eBay/FeedSDK-Python and https://github.com/eBay/ebay-oauth-python-client to python3',
    'long_description': 'Feed SDK\n==========\nPython SDK for downloading and filtering item feed files including oauth authentication.\n\nForked and merged from [https://github.com/eBay/FeedSDK-Python](https://github.com/eBay/FeedSDK-Python) and [https://github.com/eBay/ebay-oauth-python-client](https://github.com/eBay/ebay-oauth-python-client) and ported to python3\n\nNothing serious changed, made it barely working. \n\nCode is not improved yet and would need some maintenance. \n\nAutomatic Tests not working due to the nature the tests were original programmed (you need to provide actual token etc.)\n\nAvailable as PyPI package under https://pypi.org/project/ebay-feedsdk/\n\nExample code to retrieve oauth token and download file (you need working ebay-config.yaml)\n\nSee ebay_feedsk/ebay_download_example.py for example how to download feed files\n\nSee also for details:\n\n* [https://github.com/eBay/ebay-oauth-python-client/blob/master/README.adoc](https://github.com/eBay/ebay-oauth-python-client/blob/master/README.adoc)\n* [https://github.com/eBay/FeedSDK-Python/blob/master/README.md](https://github.com/eBay/FeedSDK-Python/blob/master/README.md)\n',
    'author': 'Lars Erler',
    'author_email': 'lars@xaospage.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/taxaos/FeedSDK-Python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)
